# Drupal Module Development Environment

## Contents of this file

- [Introduction](#introduction)
- [Requirements](#requirements)
- [Installation](#installation)
- [Configuration](#configuration)
- [Usage](#usage)
- [Maintainers](#maintainers)

## Introduction

This project aims to provide a lightweight development environment specifically
for developing drupal modules.

The development environment is created by using the awesome [Nix Package Manager](https://nixos.org/).

### Motivation

Developing drupal modules outside of a working drupal installation can be very cumbersome.
This development environment tries to solve this problem by creating a minimal drupal
installation directly within the module directory.

This enables the following:

- Intellisense (LSP) is working again, due to drupal being installed
- Tools like PHPCS and PHPStan work correctly
- PHPUnit tests can also be run against the drupal installation

### Kudos

I want to give kudos to the awesome [Drupalspoons Composer Plugin](https://gitlab.com/drupalspoons/composer-plugin).
This project was heavily inspired by the implementation of this composer plugin.
Shoutout to them for doing the majority of the work!

## Requirements

This project requires the [Nix Package Manager](https://nixos.org/) to be
installed on your system.

Supported systems:

- Linux (`x86_64` and `aarch64`)
- MacOS (Intel and Apple Silicon)

## Installation

Install the [Nix Package Manager](https://nixos.org/) if you haven't already.

## Configuration

In your drupal module directory (outside of a drupal installation), add a
`flake.nix` file with the following contents.

By declaring multiple `devShells`, different combinations of PHP and Drupal versions
can easily be created.

See [Usage](#usage) about how to enter a development environment for a given combination.

```nix
{
    inputs = {
        nixpkgs.url = "github:nixos/nixpkgs/nixos-24.05";

        utils.url = "github:wunderwerkio/nix-utils";
        devenv.url = "git+https://git.drupalcode.org/project/module_devenv.git?ref=1.x";
    };

    outputs = {
        self,
        nixpkgs,
        utils,
        devenv
    }: utils.lib.systems.eachDefault (system:
        let
            pkgs = import nixpkgs {
                inherit system;
            };
            mkDrupalModuleDevShell = devenv.lib.${system}.mkDrupalModuleDevShell;
        in {
            devShells = rec {
                php82_drupal10 = mkDrupalModuleDevShell {
                    # Optional supported configuration.
                    #
                    # The values assigned in this example
                    # are the default values if the option is not set.
                    drupalVersionConstraint = "^10";
                    composerFileName = "composer.devenv.json";
                    composerCacheDir = "/tmp/composer-cache";
                    webRoot = "web";
                    webPort = "9000";
                    testBaseUrl = "http://localhost:9000";
                    testDb = "sqlite://localhost/sites/default/files/.sqlite";
                    # Can be 'phpstan' or 'drupal-check'.
                    # Set to 'phpstan' if you want to use a custom phpstan.neon file.
                    analyzeTool = "drupal-check"; 

                    # Add dependencies here.
                    # Those packages are available in the devshell.
                    #
                    # PHP and Composer are required!
                    buildInputs = with pkgs; [
                        php82
                        php82Packages.composer
                    ];

                    # This shell code is run directly when entering the shell.
                    # Is optional!
                    shellHook = ''
                        echo "hello world!"
                    '';
                };

                php83_drupal11 = mkDrupalModuleDevShell {
                    drupalVersionConstraint = "^11";

                    buildInputs = with pkgs; [
                        php83
                        php83Packages.composer
                    ];
                };

                default = php83_drupal11;
            };
        }
    );
}
```

### Customize the generated composer configuration

By running `setup-drupal` a custom Composer config file is generated that
merges the root `composer.json` file and adds all additional configuration
to setup drupal within the directory.

Sometimes you may want to alter the generated composer configuration without
having to add those changes to the root `composer.json` file.

By creating a special `composer.devenv-override.json` file, you can add all
overrides there. This file is then merged alongside the `composer.json` and
the default configuration to create the resulting `composer.devenv.json` file.

### Customize the info banner

When entering the devshell, a info banner is being displayed with the available
scripts and tools.

This banner can be altered by creating a `devenv.json` file in your module root
folder. See the `./lib/default-devenv.json` file for how the JSON must look like.

**If a devenv.json file is present, it is used INSTEAD of the one supplied in
this repository! So make sure to copy the file if you want to use existing
info in your banner.**

## Usage

If everything is setup then enter a devshell by running:

```sh
nix develop # Enters the default shell -> php83_drupal11
nix develop '.#php83_drupal11' # Explicitly enters this dev shell.
nix develop '.#php82_drupal10' # Explicitly enters this dev shell.
```

When explicitly declaring the devShell, the name of a devShell (in `flake.nix`)
must be passed after `.#`.

In the devshell a info banner is being displayed with info about what scripts and
tools are available in the devshell.

To install the local Drupal installation and all required dependencies run:

```sh
setup-drupal
```

This sets everything up.  
**Run this everytime when switching versions! (by entering a different devshell)**.

Congratulations! You can now develop your module against a local Drupal installation.

E.g. you can now run `composer unit` to run unit tests.

## Maintainers

Current maintainers:

- Christoph Niedermoser ([@nimoatwoodway](https://www.drupal.org/u/nimoatwoodway))
- Christian Foidl ([@chfoidl](https://www.drupal.org/u/chfoidl))
