<?php

$cwd = getenv("SETUP_CWD");

// Safety check.
if (empty($cwd) || !is_string($cwd)) {
  throw new \Exception('Environment variable SETUP_CWD not set!');
}
if (empty(getenv("COMPOSER"))) {
  throw new \Exception('Environment variable COMPOSER not set!');
}

// Output the composer.json data as string.
// This is to be consumed by the bash script.
echo json_encode(defaultSettings());

/**
 * Get default composer.json array.
 */
function defaultSettings(): array {
  global $cwd;

  $drupalConstraint = getenv('DRUPAL_CORE_CONSTRAINT') ?: '^10';
  $webRoot = getenv('WEB_ROOT') ?: 'web';
  $webPort = getenv('WEB_PORT') ?: '8888';

  $usePhpStan = getenv('PHP_ANALYZE_TOOL') === 'phpstan';

  $config = [
    'name' => 'drupal/' . getProjectName($cwd),
    'type' => 'drupal-module',
    'description' => '',
    'keywords' => [],
    'license' => 'GPL-2.0-or-later',
    'homepage' => '',
    'authors' => [],
    'repositories' => [
      [
        'type' => 'composer',
        'url' => 'https://packages.drupal.org/8',
      ],
    ],
    'require' => [],
    'require-dev' => [
      'brianium/paratest' => '^6 || ^7',
      'composer/installers' => '^1 || ^2',
      'cweagans/composer-patches' => '^1',
      'drupal/core-composer-scaffold' => $drupalConstraint,
      'drupal/core-recommended' => $drupalConstraint,
      'drupal/core-dev' => $drupalConstraint,
      'drush/drush' => '^10 || ^11 || ^12 || ^13',
      'symfony/config' => '^6.4 || ^7'
    ],
    'minimum-stability' => 'dev',
    'scripts' => [
      'phpcs' => 'phpcs -s ' . $webRoot . '/modules/custom',
      'phpcbf' => 'phpcbf -s ' . $webRoot . '/modules/custom',
      'phpstan' => 'phpstan analyze -l2 --memory-limit=512M '. $webRoot . '/modules/custom',
      'webserver' => 'cd ' . $webRoot . ' && php -S 0.0.0.0:' . $webPort . ' .ht.router.php',
      'unit' => vsprintf('phpunit --bootstrap %s/%s/core/tests/bootstrap.php %s/%s/modules/custom', [
        $cwd,
        $webRoot,
        $cwd,
        $webRoot,
      ]),
      'unit-parallel' => vsprintf('paratest --bootstrap %s/%s/core/tests/bootstrap.php %s/%s/modules/custom', [
        $cwd,
        $webRoot,
        $cwd,
        $webRoot,
      ]),
    ],
    'config' => [
      'process-timeout' => 36000,
      "allow-plugins" => [
        "dealerdirect/phpcodesniffer-composer-installer" => true,
        "composer/installers" => true,
        "cweagans/composer-patches" => true,
        "drupal/core-composer-scaffold" => true,
        "phpstan/extension-installer" => true,
      ]
    ],
    'extra' => [
      'installer-paths' => [
        $webRoot . '/core' => [
          0 => 'type:drupal-core',
        ],
        $webRoot . '/libraries/{$name}' => [
          0 => 'type:drupal-library',
        ],
        $webRoot . '/modules/contrib/{$name}' => [
          0 => 'type:drupal-module',
        ],
        $webRoot . '/profiles/{$name}' => [
          0 => 'type:drupal-profile',
        ],
        $webRoot . '/themes/{$name}' => [
          0 => 'type:drupal-theme',
        ],
        'drush/{$name}' => [
          0 => 'type:drupal-drush',
        ],
      ],
      'drupal-scaffold' => [
        'locations' => [
          'web-root' => $webRoot . '/',
        ],
        // Do not copy these files to project-root, which in our case
        // is the module project-root folder!
        'file-mapping' => [
          '[project-root]/.editorconfig' => false,
          '[project-root]/.gitattributes' => false,
        ],
      ],
      'drush' => [
        'services' => [
          'drush.services.yml' => '^9 || ^10 || ^11',
        ],
      ],
    ],
  ];

  // Use PHPStan directly.
  if ($usePhpStan) {
    $config['scripts']['phpstan'] = 'phpstan analyze -l2 --memory-limit=512M '. $webRoot . '/modules/custom';
  }
  // Use PHPStan via drupal-check.
  else {
    $config['scripts']['phpstan'] = 'drupal-check --memory-limit=512M --drupal-root ' . $webRoot . ' ' . $webRoot . '/modules/custom/*';
    // Need to explicitly add the dependency for the binary to be available.
    $config['require-dev']['mglaman/drupal-check'] = '^1.4';
  }

  return $config;
}

/**
 * Extract the project name from the *.info.yml file.
 */
function getProjectName($projectRoot) {
  foreach (scandir($projectRoot) as $item) {
    if ($pos = strpos($item, '.info.yml')) {
      $project_name = substr($item, 0, $pos);
      break;
    }
  }
  if (!isset($project_name)) {
    throw new \Exception('The project is missing a composer.json and also the info.yml file.');
  }

  return $project_name;
}
