{
  pkgs,
  utilPkgs,
  setup-drupal,
  create-gitlab-ci-config,
  ...
}: rec {
  /*
  *
  Function to create needed exported env variables for the
  drupal module development environment.

  Returns a string with a shell script exporting the variables.

  # Usage
  Use this in the `shellHook` of your devShell!

  # Arguments (attribute set)

  drupalVersionConstraint
  : A composer version constraint for the drupal/core package
    (default: ^10)

  composerFileName
  : Filename of the special composer.json file used for the development environment.
    (default: composer.devenv.json)

  composerCacheDir
  : Directory composer uses to cache downloads and other files.
    Must be writable by your user.
    (default: /tmp/composer-cache)

  webRoot
  : Directory to install drupal to. Relative from project root.
    (default: web)

  webPort
  : Port number to use for the built-in webserver while testing.
    (default: 9000)

  testBaseUrl
  : URL used by PHPUnit browser tests to connect to.
    (default: http://localhost:${webPort})

  testDb
  : DB URI for the database.
    (default: sqlite://localhost/sites/default/files/.sqlite)

  analyzeTool
  : Which analyze tool to be used. Either "phpstan" or "drupal-check".
    Set this to "phpstan" when you need to customize the phpstan configuration.
    (default: drupal-check)
  */
  createDevShellExports = {
    drupalVersionConstraint ? "^10",
    composerFileName ? "composer.devenv.json",
    composerCacheDir ? "/tmp/composer-cache",
    webRoot ? "web",
    webPort ? "9000",
    testBaseUrl ? "http://localhost:${webPort}",
    testDb ? "sqlite://localhost/sites/default/files/.sqlite",
    analyzeTool ? "drupal-check",
    ...
  }: ''
    export NONINTERACTIVE="1"
    export COMPOSER_NO_INTERACTION="1"
    export PATH="$PATH:$(pwd)/vendor/bin"

    export DRUPAL_CORE_CONSTRAINT="${drupalVersionConstraint}"
    export COMPOSER="${composerFileName}"
    export COMPOSER_CACHE_DIR="${composerCacheDir}"
    export WEB_ROOT="${webRoot}"
    export WEB_PORT="${webPort}"
    export SIMPLETEST_BASE_URL="${testBaseUrl}"
    export SIMPLETEST_DB="${testDb}"
    export PHP_ANALYZE_TOOL="${analyzeTool}"
  '';

  /*
  *
  Function that creates a devShell with the drupal module
  development dependencies already setup.

  Otherwise usage is the same as `pkgs.mkShell`.

  Use this function instead of `pkgs.mkShell` to create a dev shell
  for drupal module development.

  # Arguments (attribute set)

  buildInputs
  : Packages to make available in the dev shell. Usage is exactly the
    same as the buildInputs in pkgs.mkShell.

  shellHook
  : Passed to the devShell shellHook. Usage is exactly the same as
    the shellHook in pkgs.mkShell.

  drupalVersionConstraint
  : A composer version constraint for the drupal/core package
    (default: ^10)

  composerFileName
  : Filename of the special composer.json file used for the development environment.
    (default: composer.devenv.json)

  composerCacheDir
  : Directory composer uses to cache downloads and other files.
    Must be writable by your user.
    (default: /tmp/composer-cache)

  webRoot
  : Directory to install drupal to. Relative from project root.
    (default: web)

  webPort
  : Port number to use for the built-in webserver while testing.
    (default: 9000)

  testBaseUrl
  : URL used by PHPUnit browser tests to connect to.
    (default: http://localhost:${webPort})

  testDb
  : DB URI for the database.
    (default: sqlite://localhost/sites/default/files/.sqlite)

  analyzeTool
  : Which analyze tool to be used. Either "phpstan" or "drupal-check".
    Set this to "phpstan" when you need to customize the phpstan configuration.
    (default: drupal-check)
  */
  mkDrupalModuleDevShell = {
    buildInputs ? [],
    shellHook ? "",
    ...
  } @ args:
    pkgs.mkShell (let
      mergedBuildInputs =
        buildInputs
        ++ [
          setup-drupal
          create-gitlab-ci-config
        ];
    in {
      buildInputs = mergedBuildInputs;

      shellHook = ''
        source ${utilPkgs.utils-sh}
        source ${utilPkgs.print-sh}
        source ${utilPkgs.devenv-sh}

        cwd=$(get_project_root)

        command_exists "php"
        if [ $? -ne 0 ]; then
          print_banner \
            --error \
            --title="Missing Requirements" \
            "PHP is not installed!" \
            "" \
            "Make sure to add your desired PHP version in buildInputs!"
          exit 1
        fi

        command_exists "composer"
        if [ $? -ne 0 ]; then
          print_banner \
            --error \
            --title="Missing Requirements" \
            "Composer is not installed!" \
            "" \
            "Make sure to add composer in buildInputs!"
          exit 1
        fi

        if [[ -z "$CI" ]]; then
          if [[ -f "$cwd/devenv.json" ]]; then
            print_info --cwd="$cwd"
          else
            print_info --cwd="${../lib}" --file="default-devenv.json"
          fi
        fi

        ${createDevShellExports args}

        ${shellHook}
      '';
    });
}
